/***********************************************************************
* Nazov suboru: FoobeSystem.cs
* Autor: Michal Pazitny
* Datum vytvorenia: 15.3.2008
* 
* Popis:  hlavna cast systemu, poskytuje vyhladavaci modul
***********************************************************************/

using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Text;


namespace Foobe
{
    //
    // trieda, ktora je "serverom" - hlavny modul, ktory obsluhuje sluzbu a riadi pracu modulov a db (v podstate iba wrapper lucene modulu, ostatne sa pouzivaju samostatne:)
    //
    public class FoobeSystem
    {
        string cestaLogSuboru="App_Data\\logFoobe.txt";  // log subor, kde sa zapisuju vykonane operacie
        string cestaKindexu="App_Data\\index";    // cesta k indexu modulu lucene
        
        LuceneModul luceneModul;    //vyhladavaci modul

        public FoobeSystem(string cestaIndex, string cestaLog)
        {
            cestaLogSuboru = cestaLog;
            cestaKindexu = cestaIndex;

            //zapisDoLogu("*** " + System.DateTime.Now + " *** Foobe spustene v " + Directory.GetCurrentDirectory() + " (" + cestaLogSuboru + ", " + cestaKindexu + ") -> ");

            vytvorLuceneModul();
        }

        // vytvorenie lucene modulu
        private int vytvorLuceneModul()
        {
            try
            {
                if (!Directory.Exists(cestaKindexu))
                    Directory.CreateDirectory(cestaKindexu);

                luceneModul = new LuceneModul(cestaKindexu);
            }
            catch (Exception e)
            {
                zapisDoLogu("chyba pri vytvarani lucene modulu\n" + e.ToString() + "\n");
                return -1;
            }

            //zapisDoLogu("ok\n");
            return 0;
        }

        // zmena adresara s indexom
        // 0 ok
        // -1 chyba
        public int zmenCestuKindexu(string novaCestaKindexu)
        {
            zapisDoLogu("*** " + System.DateTime.Now + " *** zmena adresara indexu na " + novaCestaKindexu + " -> ");
            cestaKindexu = novaCestaKindexu;
            luceneModul.zatvorModul();
            return vytvorLuceneModul();
        }

        // metoda na vratenie poctu dokumentov v indexe
        // -1 ak chyba
        public int vratPocetDokVIndexe()
        {
            int pocetDokVindexe = luceneModul.vratPocetDokVindexe();
            
            zapisDoLogu("*** " + System.DateTime.Now + " *** : vratPocetDokVIndexe -> " + pocetDokVindexe + "\n");
            
            return pocetDokVindexe;
        }

        //metoda na vratenie vysledkov hladania (dokumenty)
        public Dokument[] vyhladajDokPodlaDotazu(string dotaz)
        {
            zapisDoLogu("*** " + System.DateTime.Now + " *** : vyhladajDokPodlaDotazu\ndotaz: " + dotaz + "\n");
            
            Dokument[] najdeneDokumenty;
            najdeneDokumenty = luceneModul.vyhladajDokPodlaDotazu(dotaz);

            if (najdeneDokumenty == null)
                zapisDoLogu("-> nekorektny dotaz\n");
            else
                zapisDoLogu("-> najdenych " + najdeneDokumenty.Length + " dokumentov\n");

            return najdeneDokumenty;
        }
        
        // metoda na vyhladanie dokumentu s nejakym id
        // null nenajdeny
        public Dokument vyhladajDokPodlaId(int id)
        {
            zapisDoLogu("*** " + System.DateTime.Now + " *** : vyhladajDokPodlaId(" + id + ") -> ");

            Dokument najdenyDokument;
            najdenyDokument = luceneModul.vyhladajDokPodlaId(id);

            if (najdenyDokument == null)
                zapisDoLogu("nenajdeny\n");
            else
                zapisDoLogu("najdeny\n");

            return najdenyDokument;
        }

        //metoda na pridanie dokumentu do indexu
        // vrati id pridaneho dokumentu
        // -1 chyba pri indexacii (lucene)
        public int pridajDokDoIndexu(string nazov, string autor, string kategoria, string subor, string cestaKsuboru)
        {
            zapisDoLogu("*** " + System.DateTime.Now + " *** : pridanieDokDoIndexu ... indexovanie -> ");

            int id = -1;    //ak sa nezmeni, je to chyba pri indexacii

            try
            {
                id = luceneModul.pridajDokument(nazov, autor, kategoria, subor, cestaKsuboru);

                if (id == -1)
                    zapisDoLogu("chyba vo volani metody luceneModul.pridajDokument(...)\n"
                        + "\nnazov: " + nazov +"\n"
                        + "autor: " + autor + "\nkategoria: " + kategoria + "\n"
                        + "subor: " + subor + "\n\n");
                else    
                    zapisDoLogu("ok\npridelene id: " + id + "\nnazov: " + nazov +"\n"
                        + "autor: " + autor + "\nkategoria: " + kategoria + "\n"
                        + "subor: " + subor + "\n\n");
            }
            catch (Exception e)
            {
                zapisDoLogu("chyba Lucene\n" + e.ToString() + "\n\n");
                return -1;
            }

            return id;
        }

        // metoda na odstranie dokumentu z databazy
        // 0 ok
        // 1 nie je v indexe
        // -1 chyba
        public int odstranDokId(int id)
        {
            zapisDoLogu("*** " + System.DateTime.Now + " *** : odstranDokId " + id + " -> ");
            
            try
            {
                int vysledok = luceneModul.odstranDokId(id);
                if(vysledok == 0)
                    zapisDoLogu("ok\n");
                else if(vysledok == 1)
                    zapisDoLogu("dokument s danym id neexistuje\n");
                else
                    zapisDoLogu("chyba pri odstranovani dokumentu\n");

                return vysledok;
            }
            catch(Exception e)
            {
                zapisDoLogu("chyba\n" + e.ToString() + "\n\n"); 
                return -1;
            }
        }

        // metoda na pripisovanie do logu
        private void zapisDoLogu(string text)
        {
            File.AppendAllText(cestaLogSuboru, text);
        }
    }
}
