/***********************************************************************
* Nazov suboru: Indexer.cs
* Autor: Michal Pazitny
* Datum vytvorenia: 13.12.2007
* 
* Popis:  trieda pre zapisovanie novych dokumentov do indexu a optimalizaciu indexu
* Funkcie:  pridajDokument - pridanie dokumentu do indexu
*           optimalizujIndex - optimalizacia indexu
***********************************************************************/

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

using Lucene.Net.Documents;
using Lucene.Net.Index;
using Lucene.Net.Analysis.Standard;

namespace Foobe
{
    //
    // trieda na indexovanie dokumentov
    //
    class Indexer
    {
        string cestaIndex = "";
        IndexWriter zapisovacDoIndexu;

        // konstruktor
        public Indexer(string cesta)
        {
            cestaIndex = cesta;

            if (File.Exists(cesta + "\\segments") && File.ReadAllText(cesta + "\\segments") != "")
            {
                zapisovacDoIndexu = new IndexWriter(cestaIndex, new StandardAnalyzer(), false);
            }
            else
            {
                zapisovacDoIndexu = new IndexWriter(cestaIndex, new StandardAnalyzer(), true);
                //zapisovacDoIndexu.Close();
            }
        }
        
        // metoda na otvorenie zapisovaca do indexu
        public void otvorZapisovacDoIndexu()
        {
            zapisovacDoIndexu = new IndexWriter(cestaIndex, new StandardAnalyzer(), false);
        }

        // metoda na zatvorenie zapisovaca do indexu
        public void zatvorZapisovacDoIndexu()
        {
            zapisovacDoIndexu.Close();
        }

        // vytvorenie dokumentu
        public Document VytvorDokument(string id, string nazov, string autor, string kategoria, string subor, string text)
        {
            Document novyDokument = new Document();

            /*Console.WriteLine("ID: " + id);
            Console.WriteLine("Dokumentu: " + nazov);
            Console.WriteLine("Autor: " + autor);
            Console.WriteLine("Kategoria: " + kategoria);
            Console.WriteLine("Suboru: " + subor + "\n");*/
            
            novyDokument.Add(new Field("id", id, Field.Store.YES, Field.Index.TOKENIZED));
            novyDokument.Add(new Field("nazov", nazov, Field.Store.YES, Field.Index.TOKENIZED));
            novyDokument.Add(new Field("autor", autor, Field.Store.YES, Field.Index.TOKENIZED));
            novyDokument.Add(new Field("kategoria", kategoria, Field.Store.YES, Field.Index.TOKENIZED));
            novyDokument.Add(new Field("subor", subor, Field.Store.YES, Field.Index.NO));
            novyDokument.Add(new Field("text", text, Field.Store.NO, Field.Index.TOKENIZED));
                                    
            return novyDokument;
        }

        // pridanie dokumentu do zapisovaca
        // 0 ok
        // -1 nepodarilo sa prekonvertovat subor do textu
        public void PridajDokument(string id, string nazov, string autor, string kategoria, string subor, string cestaKsuboru)
        {
            otvorZapisovacDoIndexu();
          
            // prekonvertovanie suboru na text, ktory sa indexuje !!!
            string text = LuceneModul.konvertujDoTextu(cestaKsuboru + subor);
            //Console.WriteLine(text);
            //text = File.ReadAllText(zcestaKsuboru, Encoding.Default);

            zapisovacDoIndexu.AddDocument(VytvorDokument(id, nazov, autor, kategoria, subor, text));
            zatvorZapisovacDoIndexu();
        }

        // optimalizacia indexu
        public void optimalizujIndex()
        {
            Console.WriteLine("Optimalizujem index...\n");
            otvorZapisovacDoIndexu();
            zapisovacDoIndexu.Optimize();
            zatvorZapisovacDoIndexu();
            Console.WriteLine("Index uspesne optimalizovany\n");
            Console.WriteLine("*********************************************************\n");
        }
    }
}
