/***********************************************************************
* Nazov suboru: LuceneModul.cs
* Autor: Michal Pazitny
* Datum vytvorenia: 13.12.2007
* 
* Popis:  trieda pre pracu s indexom - pridavanie a odstranovanie dokumentov, vyhladavanie
***********************************************************************/

using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

using Lucene.Net.Index;
using Lucene.Net.Documents;
using Seekafile.Plugin.Default;

namespace Foobe
{
    //
    // vyhladavaci modul
    //
    public class LuceneModul
    {
        string cestaIndex = "";
        Indexer indexer;
        Vyhladavac vyhladavac;
        const int MAX = 15000;  // maximalny pocet dokumentov v indexe

        // konstruktor
        public LuceneModul(string cesta)
        {
            cestaIndex = cesta;
            indexer = new Indexer(cestaIndex);
            vyhladavac = new Vyhladavac(cestaIndex);
        }

        // destruktor
        public void zatvorModul()
        {
            indexer.zatvorZapisovacDoIndexu();
            vyhladavac.zatvor();
        }

        // pridanie dokumentu
        public int pridajDokument(string nazov, string autor, string kategoria, string subor, string cestaKsuboru)
        {
            Random random = new Random();
            int id = random.Next(MAX);  // ak sa ocakava viac suborov v databaze, je nutne lepsie id-ckovanie

            //najdenie cisla ktore nie je v databaze
            vyhladavac.obnovVyhladavac();
            while (vyhladavac.existujeId(id.ToString())) id = random.Next();

            try
            {
                Console.WriteLine("Pridavam dokument...\n");
                indexer.PridajDokument(id.ToString(), nazov, autor, kategoria, subor, cestaKsuboru);
                indexer.optimalizujIndex();
                Console.WriteLine("Dokument bol uspesne pridany\n");
                Console.WriteLine("*********************************************************\n");
            }
            catch (Exception e)
            {
                Console.WriteLine("Neuspesne pridanie!!!\n" + e.ToString());
                return -1;
            }

            return id;
        }

        // odstranenie dokumentu s prislusnym id
        // 0 ok
        // 1 nie je id v indexe
        // -1 chyba
        public int odstranDokId(int id)
        {
            int odstranenie = vyhladavac.odstranDokId(id.ToString());

            if(odstranenie == 0)
            {
                Console.WriteLine("Dokument s id " + id + " uspesne odstraneny.");
                
                //----> preindexovanie + odstranenie z databazy
                indexer.optimalizujIndex();
                vyhladavac.obnovVyhladavac();
            }
            return odstranenie;
        }

        // vyhladanie dokumentov podla dotazu
        public Dokument[] vyhladajDokPodlaDotazu(string dotaz)
        {
            /*Dokument[] najdeneDokumenty;
            najdeneDokumenty = vyhladavac.vyhladajDokPodlaDotazu(dotaz);
            return najdeneDokumenty;  // ----> vrati null ak nastala chyba*/
            
            return vyhladavac.vyhladajDokPodlaDotazu(dotaz); // ----> vrati null ak nastala chyba
        }

        public Dokument vyhladajDokPodlaId(int id)
        {
            return vyhladavac.vyhladajDokPodlaId(id.ToString());
        }

        // vrati pocet dokumentov v indexe
        public int vratPocetDokVindexe()
        {
            return vyhladavac.vratPocetDokVindexe();
        }

        // vypisanie cesty k indexu
        public string vratCestuKindexu()
        {
            return cestaIndex;
        }

        // optimalizcia indexu
        public void optimalizujIndex()
        {
            indexer.optimalizujIndex();
        }

        // konvertovanie do sucheho textu
        // trieda ifilter DOC, DOCX, XLS, XLSX, PDF, PPT, HTM, HTML, TXT
        // trieda rtfplugin RTF
        public static string konvertujDoTextu(string cestaKsuboru)
        {
            FileInfo subor = new FileInfo(cestaKsuboru);
            
            if (subor.Extension == ".rtf")
            {
                RtfPlugin a = new RtfPlugin();
                return a.getBody(cestaKsuboru);
            }
                       
            return IFilterPlugin.Extract(cestaKsuboru);
        }
    }
}