/***********************************************************************
* Nazov suboru: Vyhladavac.cs
* Autor: Michal Pazitny
* Datum vytvorenia: 13.12.2007
* 
* Popis:  vyhladavanie dokumentov podla id, dotazu
*         odstranovanie dokumentov
***********************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

using Lucene.Net.Search;
using Lucene.Net.QueryParsers;
using Lucene.Net.Documents;
using Lucene.Net.Analysis.Standard;
using Lucene.Net.Index;

namespace Foobe
{
    //
    // trieda na vyhladavanie
    // 
    class Vyhladavac
    {
        string cestaIndex;
        IndexSearcher vyhladavacVindexe;
        QueryParser parser;

        // konstrukor
        public Vyhladavac(string cesta)
        {
            cestaIndex = cesta;
            try
            {
                vyhladavacVindexe = new IndexSearcher(cestaIndex);
            }
            catch (Exception e)
            { Console.WriteLine(e.ToString()); }
        }

        // metoda pre otvorenie vyhladavaca
        public void obnovVyhladavac()
        {
            if (vyhladavacVindexe.GetIndexReader().IsCurrent()) return;
            vyhladavacVindexe = new IndexSearcher(cestaIndex);
        }

        // destrukcia
        public void zatvor()
        {
            try
            {
                vyhladavacVindexe.Close();
            }
            catch (System.IO.IOException) { } //uz je zatvoreny
        }
        
        // test ci v indexe existuje dokument s nejakym id
        public Boolean existujeId(string id)
        {
            if (!(id is string))
            {
                Console.WriteLine("Nekorektne ID\n");
                return false;
            }
          
            obnovVyhladavac();

            parser = new QueryParser("id", new StandardAnalyzer());
            Query query = parser.Parse(id);
            Hits hits = vyhladavacVindexe.Search(query);

            if (hits.Length() == 0) return false;
            if (hits.Length() == 1) return true;

            Console.WriteLine("*******************************************************");
            Console.WriteLine("* Dva rovnake ID v indexe!!! - potreba preindexovania *");
            Console.WriteLine("*******************************************************");
            return false;
        }

        // vyhladanie dokumentu s prislusnym id (moze byt iba jeden)
        public Dokument vyhladajDokPodlaId(string id)
        {
            obnovVyhladavac();

            parser = new QueryParser("id", new StandardAnalyzer());
            Query query = parser.Parse(id);
            Hits hits = vyhladavacVindexe.Search(query);

            if (hits.Length() == 0) return null;
            if (hits.Length() > 1)
            {
              Console.WriteLine("*******************************************************");
              Console.WriteLine("* Dva rovnake ID v indexe!!! - potreba preindexovania *");
              Console.WriteLine("*******************************************************");
            }
            
            // ak existuje dokument s danym id            
            Dokument dokument = new Dokument(int.Parse(hits.Doc(0).Get("id").ToString()),
                              hits.Doc(0).Get("nazov").ToString(),
                              hits.Doc(0).Get("subor").ToString(),
                              hits.Doc(0).Get("autor").ToString(),
                              hits.Doc(0).Get("kategoria").ToString(),
                              hits.Score(0));      
            return dokument;
        }

        // odstranenie dokumentu s danym id
        // 0 ok
        // 1 dane id nie je v indexe
        // -1 chyba
        public int odstranDokId(string id)
        {
            obnovVyhladavac();

            parser = new QueryParser("id", new StandardAnalyzer());
            Query query = parser.Parse(id);
            Hits hits = vyhladavacVindexe.Search(query);

            if (hits.Length() == 0)
            {
              Console.WriteLine("Dokument s ID " + id + " nie je v indexe\n");
              return 1;
            }

            int idVlucene = hits.Id(0);

            IndexReader indexReader = IndexReader.Open(cestaIndex);
            indexReader.DeleteDocument(idVlucene);
            Console.WriteLine("Dokument s LuceneID " + idVlucene + " vymazany: "+ indexReader.IsDeleted(idVlucene));
            if (indexReader.IsDeleted(idVlucene))
            {
                try { indexReader.Close(); }
                catch (UnauthorizedAccessException) { }
                return 0;
            }
            indexReader.Close();
            return -1;
        }

        // vyhladanie dokumentov podla vyhladavacieho dotazu
        public Dokument[] vyhladajDokPodlaDotazu(string dotaz)
        {
            obnovVyhladavac();

            parser = new QueryParser("text", new StandardAnalyzer());
            try
            {
                Query query = parser.Parse(dotaz);
                Hits hits = vyhladavacVindexe.Search(query);

                Dokument[] dokumenty = new Dokument[hits.Length()];
                Document tmp;
                for (int i = 0; i < hits.Length(); i++)
                {
                    tmp = hits.Doc(i);
                    dokumenty[i] = new Dokument(int.Parse(tmp.Get("id").ToString()),
                              tmp.Get("nazov").ToString(),
                              tmp.Get("subor").ToString(),
                              tmp.Get("autor").ToString(),
                              tmp.Get("kategoria").ToString(),
                              hits.Score(i));
                }
                return dokumenty;
            }
            catch (Lucene.Net.QueryParsers.ParseException)
            {
                Console.WriteLine("Nekorektny dotaz!\n");
                return null;
            }
        }

        // vrati pocet dokumentov v indexe
        public int vratPocetDokVindexe()
        {
            return vyhladavacVindexe.GetIndexReader().NumDocs();
        }
    }
}
