/***********************************************************************
* Nazov suboru: Prepojenia.cs
* Autor: luksa sim
* Datum vytvorenia: 28.5.2008
* 
* Popis:  dokumetn obsahujuci triedu pre vyhladavanie dokumentov prostrednictvom ich prepojeni
* Funkcie:  najdenie prepojeni medzi dokumentami, ich zapisanie do databazy, najdenie dokumentov prostrednictvom 
 *          prepojeni
 *Spolupraca : modul spolupracuje s modulom dot.lucene a s databazovym modulom
***********************************************************************/

using System;
using System.Collections.Generic;
using System.Text;


namespace Foobe
{
    //pomocna treida v ktorej sa ukladaju dokumenty s ich vyslednou relevanciou

    public class Prepojenia
    {
        /*private String luceneString = "asdf";
        private String server_name = "xcipak\\sqlexpress";
        private String db_name = "test";
        private String username = "foobe";
        private String password = "a";
        */

        private float vaha_nazov = 0.9f;
        private float vaha_autor = 0.5f;
        private float vaha_subor = 0.3f;

        //pocet dokumentov ktore sa budu brat na zaciatku hladania
        private int pocet_zaciatocnych = 10;
        /*
         * Typy prepojeni
         * podla nazvu dokumentu    1
         * podla mena autora        2
         * podla nazvu suboru       3
        */

        /**
         * funkcia najde prepojenia medzi zadanym dokumentom a inymi dokumentami ktore su zaindexovane
         * tieto prepojenia potom ulozi do databazy
         * **/
        public void najdiPrepojenia(Dokument dokument, FoobeSystem foobeSystem, DB databaza)
        {

            /*bool debug = true;
            DB databaza = new DB();
            databaza.pripojSa(server_name, db_name, username, password);
            */
            ////najdu sa dokumenty obsahujuce nazov noveho dokumentu
            Dokument[] najdene_dokumenty;

            najdene_dokumenty = foobeSystem.vyhladajDokPodlaDotazu(dokument.VratNazov());

            if (najdene_dokumenty == null) return;  // nekorektny dotaz

            int poc = najdene_dokumenty.Length;
            int i;
            for (i = 0; i < poc; i++)
            {
                if (dokument.VratID() == najdene_dokumenty[i].VratID()) continue;
                databaza.pridajPrepojenie(dokument.VratID(), najdene_dokumenty[i].VratID(), 1, najdene_dokumenty[i].VratRelevanciu());
                databaza.pridajPrepojenie(najdene_dokumenty[i].VratID(), dokument.VratID(), 1, najdene_dokumenty[i].VratRelevanciu());
                //if (debug) Console.WriteLine("pridavam prepojenie na zaklade nazvu dokumentu");
            }

            // podla autorov - viaceri oddeleni ciarkami, nutne vyhladavat osobitne
            string[] autori = dokument.VratAutora().Split(',');
            for (int cisloAutora = 0; cisloAutora < autori.Length; cisloAutora++)
            {
                najdene_dokumenty = foobeSystem.vyhladajDokPodlaDotazu("\"" + autori[cisloAutora].Trim() + "\"");   // dotaz: "meno priezvisko"
                poc = najdene_dokumenty.Length;
                for (i = 0; i < poc; i++)
                {
                    if (dokument.VratID() == najdene_dokumenty[i].VratID()) continue;
                    databaza.pridajPrepojenie(dokument.VratID(), najdene_dokumenty[i].VratID(), 2, najdene_dokumenty[i].VratRelevanciu());
                    databaza.pridajPrepojenie(najdene_dokumenty[i].VratID(), dokument.VratID(), 2, najdene_dokumenty[i].VratRelevanciu());
                    //if (debug) Console.WriteLine("pridavam prepojenie na zaklade autora");
                }
            }

            najdene_dokumenty = foobeSystem.vyhladajDokPodlaDotazu(dokument.VratSubor());
            poc = najdene_dokumenty.Length;
            for (i = 0; i < poc; i++)
            {
                if (dokument.VratID() == najdene_dokumenty[i].VratID()) continue;
                databaza.pridajPrepojenie(dokument.VratID(), najdene_dokumenty[i].VratID(), 3, najdene_dokumenty[i].VratRelevanciu());
                databaza.pridajPrepojenie(najdene_dokumenty[i].VratID(), dokument.VratID(), 3, najdene_dokumenty[i].VratRelevanciu());
                //if (debug) Console.WriteLine("pridavam prepojenie na zaklade nazvu suboru");
            }
            
        }

        /***
         * Funkcia vrati dokumenty najdene ku klucovym slovam na zaklade vyskytu prepojeni s inymi dokumentami
         * 
         * ***/

        public VysledokModulu[] vratDokumenty(Dokument[] zaciatocne_dokumenty, DB databaza)
        {

            VysledokModulu[] vratit = null;
            int i;
            
            //DB databaza = new DB();
            //databaza.pripojSa(server_name, db_name, username, password);
            
            int pocet = 0;

            if (pocet_zaciatocnych < zaciatocne_dokumenty.Length) pocet = pocet_zaciatocnych;
            else pocet = zaciatocne_dokumenty.Length;

            LinkedList<VysledokModulu> zoznam = new LinkedList<VysledokModulu>();

            //pre kazdy zo zaciatocnych dokumentov najdeme dokumenty s ktorymi je prepojeny
            for (i = 0; i < pocet; i++)
            {
                VysledokModulu pom;

                Prepojenie[] prepojenia  = databaza.vratPrepojenia(zaciatocne_dokumenty[i].VratID());

                int j;

                for (j = 0; j < prepojenia.Length; j++)
                {
                    pom = new VysledokModulu();
                    pom.id = prepojenia[j].dokument_ID;

                    switch (prepojenia[j].typ)
                    {
                        //prepojenie na zaklade nazvu dokument
                        case 1:
                            pom.relevancia = (float)prepojenia[j].relevancia * vaha_nazov;
                            break;
                        case 2:
                            pom.relevancia = (float)prepojenia[j].relevancia * vaha_autor;
                            break;
                        case 3:
                            pom.relevancia = (float)prepojenia[j].relevancia * vaha_subor;
                            break;
                    }

                    zoznam.AddLast(pom);
                }
            }

            vratit = new VysledokModulu[zoznam.Count];
            zoznam.CopyTo(vratit, 0);
            
            return vratit;
        }
    }
}
