/*
Copyright  2006 - 2007 Annpoint, s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

-------------------------------------------------------------------------

NOTE: Reuse requires the following acknowledgement (see also NOTICE):
This product includes Seekafile Server (http://www.seekafile.org) 
developed by Annpoint, s.r.o.

*/


using System.IO;
using Lucene.Net.Documents;
using Seekafile.Plugin;
using Seekafile.Plugin.Default;

namespace Seekafile.Plugin.Default
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class FileInfoPlugin : IDocumentPlugin
	{
		private string[] extensions = { ".*" };
		private string description = "Indexes only the basic file information (name, path, size, last modified date, etc.).";

		#region IDocumentPlugin Members

		public Lucene.Net.Analysis.Analyzer Analyzer(string path)
		{
			return null;
		}

		public string Description
		{
			get { return description; }
		}

		public string[] Extensions
		{
			get { return extensions; }
		}

		public Lucene.Net.Documents.Document Document(string path)
		{
			FileInfo fi = new FileInfo(path);
			Document doc = new Document();
			doc.Add(new Field("name", fi.Name, true, true, true));
			doc.Add(new Field("fullname", fi.FullName, true, true, false));
	
			DirectoryInfo di = fi.Directory;
			doc.Add(new Field("directparent", di.FullName, true, true, false));
			while (di != null)
			{
				doc.Add(new Field("parent", di.FullName, true, true, false));
				di = di.Parent;
			}
	
			doc.Add(Field.Keyword("created", DateField.DateToString(fi.CreationTime)));
			doc.Add(Field.Keyword("modified", DateField.DateToString(fi.LastWriteTime)));
			doc.Add(Field.Keyword("length", fi.Length.ToString()));
			doc.Add(Field.Keyword("extension", fi.Extension));

			// we are not adding "body" field

			return doc;
		}

		#endregion
	}
}
