/*
Copyright  2006 - 2007 Annpoint, s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

-------------------------------------------------------------------------

NOTE: Reuse requires the following acknowledgement (see also NOTICE):
This product includes Seekafile Server (http://www.seekafile.org) 
developed by Annpoint, s.r.o.

*/

using System;
using System.IO;
using System.Windows.Forms;
using Lucene.Net.Analysis;
using Lucene.Net.Documents;
using Seekafile.Plugin;

namespace Seekafile.Plugin.Default
{
	/// <summary>
	/// Parses RTF files.
	/// </summary>
	public class RtfPlugin : IDocumentPlugin
	{
		public RtfPlugin()
		{
		}
		#region IParserPlugin Members

		/// <summary>
		/// This is the main parsing method. It extracts the plain text from a file.
		/// </summary>
		/// <param name="path">Full path to the file.</param>
		/// <returns>Content of the file as plain text.</returns>
		public string getBody(string path)
		{
			RichTextBox rtb = new RichTextBox();
			rtb.LoadFile(path);
			return rtb.Text;
		}

		#endregion

		public Document Document(string path)
		{
			FileInfo fi = new FileInfo(path);
			Document doc = new Document();
			doc.Add(new Field("name", fi.Name, true, true, true));
			doc.Add(new Field("fullname", fi.FullName, true, true, false));
	
			DirectoryInfo di = fi.Directory;
			doc.Add(new Field("directparent", di.FullName, true, true, false));
			while (di != null)
			{
				doc.Add(new Field("parent", di.FullName, true, true, false));
				di = di.Parent;
			}
	
			doc.Add(Field.Keyword("created", DateField.DateToString(fi.CreationTime)));
			doc.Add(Field.Keyword("modified", DateField.DateToString(fi.LastWriteTime)));
			doc.Add(Field.Keyword("length", fi.Length.ToString()));
			doc.Add(Field.Keyword("extension", fi.Extension));
	
			string rawText = getBody(path);
			doc.Add(new Field("body", rawText, true, true, true));
			return doc;
		}

		public Analyzer Analyzer(string path)
		{
			return null;
		}

		public string Description
		{
			get { 
				return "Parses RTF files.";
			}
		}

		public string[] Extensions
		{
			get
			{
				return new string[] { ".rtf" };
			}
		}
	}
}
