/***********************************************************************
* Nazov suboru: Recommender.cs
* Autor: Daniel Princzkel
* Datum vytvorenia: 8.4.2008
* 
* Popis:  trieda pre navrh relevantnych dokumentov pre aktualneho uzivatela
* Funkcie:  
***********************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Data.SqlClient;
using System.Data;

namespace Foobe
{
    //Poradca pre pouzivatela(socialne siete)
    public class Recommender
    {
        
        /******************************************************
        * 
        *                  ATRIBUTY
        * 
        ******************************************************/
         
        //id aktualneho pouzivatela
        public int userId;
        
        //priemerne hodnotenie bazy aktualneho pouzivatela
        public double priemUserId; // = priemerneHodnotenie(userId);

        //blizkost s ostatnymi pouzivatelmi
        public double[] relatedUser;

        //pole skutocnych(z DB) id dokumentov
        public int[] docIds;

        //pole skutocnych(z DB) id puzivatelov
        public int[] userIds;

        //kategorie_dokumentov
        public int[] kategorie;

        //podobne dokumenty
        public double[] relatedDocs;

        //ohodnotenia dokumentov uzivatelmi - pocet otvoreni
        public int[][] UDH;
        
        //vysledky vyhladania
        public Vysledok[] vysledky;


        /*******************************************************
        *
        *                   KONSTRUKTORY
        * 
        *******************************************************/

        public Recommender()
        { 
        }

        //public Recommender(int userId, Vysledok[] vysledky, string connectionString)
        public Recommender(int userId, string connectionString)
        {
            int n_docs;
            int n_users;
            int pom;
            SqlDataReader reader;
            SqlCommand command;

            SqlConnection connection = new SqlConnection(connectionString);/*@"Data source = danielson\sqlexpress;" +
                "database = foobeDB; User ID =foobe; Password =a;");*/
      
            connection.Open();

            command = connection.CreateCommand();
            command.CommandText = "Select COUNT(distinct ID_pouzivatel) from pocetnostOtvoreni";
            reader = command.ExecuteReader();
            reader.Read();
            n_users = (int)reader[0];

            connection.Close();

            connection.Open();

            command = connection.CreateCommand();
            command.CommandText = "Select COUNT(distinct ID_dokument) from pocetnostOtvoreni";
            reader = command.ExecuteReader();
            reader.Read();
            n_docs = (int)reader[0];

            connection.Close();

            //this.vysledky = vysledky;
            this.userIds = new int[n_users];
            this.docIds = new int[n_docs];
            this.kategorie = new int[n_docs];
            this.relatedDocs = new double[n_docs];
            this.relatedUser = new double[n_users];
            this.UDH = new int[n_docs][];
            for (int i = 0; i < n_docs; i++ )
                UDH[i] = new int[n_users];

            connection.Open();

            command = connection.CreateCommand();
            command.CommandText = "Select distinct ID_pouzivatel from pocetnostOtvoreni";
            reader = command.ExecuteReader();
            pom = 0;
            while (reader.Read())
                userIds[pom++] = (int)reader[0];

            connection.Close();
            
            connection.Open();

            command = connection.CreateCommand();
            command.CommandText = "Select distinct ID_dokument from pocetnostOtvoreni";
            reader = command.ExecuteReader();
            pom = 0;
            while (reader.Read())
                docIds[pom++] = (int)reader[0];

            connection.Close();

            /*connection.Open();

            command = connection.CreateCommand();
            command.CommandText = "Select kategoria from subor";
            reader = command.ExecuteReader();
            pom = 0;
            while (reader.Read())
                kategorie[pom++] = (int)reader[0];

            connection.Close();
            */
    
            connection.Open();

            command = connection.CreateCommand();
            command.CommandText = "Select * from pocetnostOtvoreni";
            reader = command.ExecuteReader();
            while (reader.Read())
                UDH[getID((int)reader[1], docIds)][getID((int)reader[0], userIds)] = (int)reader[2];
            
            connection.Close();
            

            this.userId = getID(userId, userIds);

            // ak pouzivatel nie je v tabulke pocetnosti (nic zatial nestahoval), tak 
            //---> TODO if(this.userId) return null

            if(this.userId != -1) priemUserId = priemerneHodnotenie(this.userId);            
        }
      
        /*******************************************************
        *
        *                   FUNKCIE
        * 
        *******************************************************/
        
        /*******************************************************
        *  tu sa zistuje korelacia dvoch pouzivatelov, aktualneho a druheho - otestovane
        *******************************************************/ 
        public double PearsonCorel(int druhyId) 
        {
            //ak ten isty pouzivatel;
            if (userId == druhyId)
                return 0;
            
            double priemDruhyId = priemerneHodnotenie(druhyId);

            //pomocna premenna = UDH[k][userId] - priemUserId
            double a;
 
            //pomocna premenna = UDH[k][druhyId] - priemDruhyId
            double b;

            //citatel vzorca
            double citatel = 0;

            //menovatele vzorca
            double menovatel_1 = 0;
            double menovatel_2 = 0;

            //ak priemerne hodnotenie pre jedneho uziv je nulove, koncime
            if(priemUserId==0 || priemDruhyId==0)
                return 0;

            //pre vsetky dokumenty v baze
            for (int k = 0; k < UDH.Length; k++) 
            { 
                //ak nehodnotili obaja, preskoc takyto dokument
                if (UDH[k][userId] == 0 || UDH[k][druhyId] == 0)
                    continue;

                a = UDH[k][userId] - priemUserId;
                b = UDH[k][druhyId] - priemDruhyId;

                citatel += a * b;
                menovatel_1 += a * a;
                menovatel_2 += b * b;
            }

            //aby sme dostali pozitivny vysledok
            if (citatel < 0) 
                citatel = (-1) * citatel;

            if (menovatel_1 == 0 || menovatel_2 == 0)
                return 0;
            else return (citatel / Math.Sqrt(menovatel_1 * menovatel_2));

        }

        /*******************************************************
        *  priemerne hodnotenie pouzivatela z historie - otestovane
        *******************************************************/
        public double priemerneHodnotenie(int uzivatel) 
        {
            //pocet ohodnotenych dokumentov
            double pocet = 0;
            //suma ohodnoteni
            double suma = 0;

            if (UDH == null)
                return -1;

            //prejdi vsetky dokumenty
            for (int i = 0; i < UDH.Length; i++)
            {
                if (UDH[i][uzivatel] != 0)
                    suma += UDH[i][uzivatel];
                pocet++;
            }
            
            //vrat priemer
            if (pocet > 0)
                return suma / pocet;  
            else return 0;
        }

        /*******************************************************
        *  zistenie podobnosti aktualneho pouzivatela s ostatnymi - otestovane
        *******************************************************/ 
        public void getRelatedUser() 
        {
            if (relatedUser == null)
                return;
            for (int i = 0; i < relatedUser.Length; i++) 
            {
                if (i == userId)
                {
                    relatedUser[i] = 0;
                    continue;
                }
                else 
                {
                    relatedUser[i] = PearsonCorel(i);
                }
            }
        }

        /*******************************************************
        *  navrh najpravdepodobnejsieho dokumentu k druhemu - docId
        *  uMAX - pocet najblizsich susedov - otestovane, ale tu by sa dalo vybrat mnoho inych techn.
        *******************************************************/
        public void sortDocs(int uMAX, int[] vysledky)
        {
            if (uMAX > userIds.Length - 1)
                uMAX = userIds.Length - 1;
            
            //zistenie korelacie pouzivatelov
            getRelatedUser();
            //zoradenie pouzivatelov podla blizkosti
            int[] zoradeni = insertionSort(relatedUser);
            
            // pre vsetky dok
            for (int d = 0; d < UDH.Length; d++)
            {
                relatedDocs[d] = 0;
                //pre najlepsie vysledky(zoredene podla relevancie) vyhladania
                for (int v = 0; v < vysledky.Length; v++)
                {
                    double pom = 0;
                    //ak sa porovnava vysledok so sebou
                    if (d == v) continue;
                    //a teraz najdi prvych k najblizsich susedov a vyber najpodobnejsie dokumenty z ich historie
                    for (int i = 0; i < uMAX; i++)
                    {
                        if (UDH[d][zoradeni[i]] == 0) continue;
                        else
                        {
                             pom += UDH[d][zoradeni[i]];
                        }
                        pom *= getDocRelation(d, v);
                    }
                    relatedDocs[d] += pom;
                }
            }
            
            
        }

        /*******************************************************
        *  navrh najpravdepodobnejsieho dokumentu k druhemu - docId
        *  uMAX - pocet najblizsich susedov - otestovane, ale tu by sa dalo vybrat mnoho inych techn.
        *******************************************************/
        public void sortDocs(int uMAX)
        {
            if (uMAX > userIds.Length - 1)
                uMAX = userIds.Length - 1;
            double suma = 0;
            
            //zistenie korelacie pouzivatelov
            getRelatedUser();
            
            //zoradenie pouzivatelov podla blizkosti
            int[] zoradeni = insertionSort(relatedUser);

            // pre vsetky dok
            for (int d = 0; d < UDH.Length; d++)
            {
                relatedDocs[d] = 0;
                
                //a teraz najdi prvych k najblizsich susedov a vyber najpodobnejsie dokumenty z ich historie
                if (UDH[d][userId] > 0)
                    continue;
                for (int i = 0; i < uMAX; i++)
                {
                    
                    double x = relatedUser[i] * UDH[d][zoradeni[i]];
                    relatedDocs[d] += x;
                    suma += x;
                }   
            }
            for (int d = 0; d < UDH.Length; d++)
            {
                relatedDocs[d] /= suma;
            }
            
        }

        /*******************************************************
        *  zistenie podobnosti dvoch dokumentov
        *******************************************************/ 
        public double getDocRelation(int i, int j) 
        {
            return 1;
        }

        /*******************************************************
        *  Navrh relevantnych dokumentov k vysledkom, ich pocet je N 
        *******************************************************/
        public VysledokModulu[] RecommendDocs(int MAX, int uMax)
        {
            if (userId == -1)
                return new VysledokModulu[0];
            int max;
            
            int[] indexy = new int[relatedDocs.Length];
            for (int i = 0; i < indexy.Length; i++)
                indexy[i] = i;
            
            if (MAX > relatedDocs.Length) 
                MAX = relatedDocs.Length;
            
            VysledokModulu[] noveVysl = new VysledokModulu[MAX];

            /*
            //najdi pat najlepsich vysledkov
            int[] zoradeneVysledky  = findFirstN(vysledky, 5);
            //vyhodnot vsetky dokumenty podla 3 najblzsich uzivatelov 
            sortDocs(uMax, zoradeneVysledky);
            */

            sortDocs(uMax);
            
            //sortovanie MAX najlepsich
            for (int i = 0; i < MAX; i++)
            {
                max = i;
                //najdi max od
                for (int j = relatedDocs.Length - 1; j > i; j--)
                    if (relatedDocs[j] > relatedDocs[max]) max = j;

                if (max != i)
                {
                    double pom = relatedDocs[max];
                    relatedDocs[max] = relatedDocs[i];
                    relatedDocs[i] = pom;

                    int p = indexy[max];
                    indexy[max] = indexy[i];
                    indexy[i] = p;
                }

                
                noveVysl[i] = new VysledokModulu();
                noveVysl[i].id = docIds[indexy[i]];
                noveVysl[i].relevancia = (float)relatedDocs[i]; 
            }

            int pocet=0;
            for (int i = noveVysl.Length - 1; i >= 0; i--)
            {
                if (noveVysl[i].relevancia == 0)
                    pocet++;
                else break;
            }
            VysledokModulu[] vysledky = new VysledokModulu[noveVysl.Length - pocet];
            for (int i = 0; i < vysledky.Length; i++)
                vysledky[i] = noveVysl[i];
            
            return vysledky;
        }

        /*******************************************************
        *  Zorad vysledky a vrat prvych n -- otestovane
        *******************************************************/
        public int[] findFirstN(Vysledok[] vysledky, int N) 
        { 
            
            int max;
            if(N > vysledky.Length)
                N = vysledky.Length;
            int[] noveVysl = new int[N];
            
            //pre vsetky prvky noveho pola
            for(int i=0; i<N; i++)
            {
                max = i;
                //najdi max od
                for (int j = vysledky.Length - 1; j > i; j--)
                    if (vysledky[j].relevancia > vysledky[max].relevancia) max = j;

                if (max != i)
                {
                    Vysledok pom = vysledky[max];
                    vysledky[max] = vysledky[i];
                    vysledky[i] = pom;
                }
                noveVysl[i] = getID(vysledky[i].id, docIds);
            }

            return noveVysl;
            /*
            Vysledok[] navrh = new Vysledok[N];
            
            SqlDataReader reader;
            SqlCommand command;

            SqlConnection connection = new SqlConnection(@"Data source = danielson\sqlexpress;" +
                "database = DB; User ID =foobe; Password =a;");

            for (int i = 0; i < N; i++)
            {
                navrh[i] = new Vysledok();
                connection.Open();
                command = connection.CreateCommand();
                command.CommandText = "Select COUNT(*) from subor WHERE ID = ";
                reader = command.ExecuteReader();
                reader.Read();
                navrh[i].id = (int)reader[0];
                connection.Close();
            }*/
        }

        /*******************************************************
        *  insertSort vrati zoznam preusporiadania indexov --otestovane
        *******************************************************/
        public static int[] insertionSort(double[] numbers)
        {
            int i, j;
            int ind;
            double index;

            int[] id = new int[numbers.Length];
            for (int k = 0; k < numbers.Length; k++)
                id[k] = k;

            for (i = 1; i < numbers.Length; i++)
            {
                index = numbers[i];
                ind = id[i];
                j = i;
                while ((j > 0) && (numbers[j - 1] < index))
                {
                    numbers[j] = numbers[j - 1];
                    id[j] = id[j - 1];
                    j = j - 1;
                }
                numbers[j] = index;
                id[j] = ind;
            }

            return id;
        }

        /*******************************************************
        *  Zo skutocneho id(v DB) zistime poradie v DB tabulke --otestovane
        *******************************************************/
        public int getID(int value, int[] pole)
        {
            for (int i = 0; i < pole.Length; i++)
            {
                if (pole[i] == value)
                    return i;
            }
            return -1;
        }

    }

}
