/***********************************************************************
* Nazov suboru: FoobeServis.cs
* Autor: Michal Pazitny
* Datum vytvorenia: 4.4.2008
* 
* Popis:  sluzba, ktora poskytuje funkcie znalostneho manazmentu
***********************************************************************/

using System;
using System.Configuration;
using System.IO;
using System.Security.Cryptography;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using Foobe;

using System.Xml;
using System.Collections.Generic;

[WebService(Namespace = "FoobeServis")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
public class FoobeServis : System.Web.Services.WebService
{
    FoobeSystem foobe;      //instancia systemu foobe
    DB db = new DB();       //databaza

    string cestaLogSuboru;  // log subor, kde sa zapisuju vykonane operacie
    string cestaLogFoobe;   // log subor, kde sa zapisuju vykonane operacie
    string adresarIndex;    // log subor, kde sa zapisuju vykonane operacie


    public FoobeServis()
    {
        cestaLogSuboru = "App_Data\\logServis.txt";    // protokolovy subor
        adresarIndex = ConfigurationManager.AppSettings["adresarIndex"]; //cesta k indexu vyhladavacieho modulu

        cestaLogFoobe = "App_Data\\" + ConfigurationManager.AppSettings["logFoobe"].ToString();

        foobe = new FoobeSystem("App_Data\\" + adresarIndex, cestaLogFoobe);
        
        try
        {
            db.pripojSa(ConfigurationManager.AppSettings["dbServer"],
                    ConfigurationManager.AppSettings["dbNazov"],
                    ConfigurationManager.AppSettings["dbLogin"],
                    ConfigurationManager.AppSettings["dbHeslo"]);
        }
        catch (Exception e)
        {
            zapisDoLogu("nepodarilo sa nadviazat spojenie s databazou\n" +
                    "server -> " + ConfigurationManager.AppSettings["dbServer"] +
                    ", nazov -> " + ConfigurationManager.AppSettings["dbNazov"] + "\n" +
                    e.ToString() + "\n");
        }

        //zapisDoLogu(DateTime.Now + " -> servis spusteny\n");
    }

    //
    //******* testovanie servisu, nastavenia log suborov a index ****************************//
    //

    //otestovanie sluzby, ci je v poriadku komunikacia s databazou a vyhladavacom
    [WebMethod]
    public string testServisu()
    {
        string odpoved;

        odpoved = DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : testServisu -> servis Foobe je funkcny\n";
        odpoved += "server: " + HttpContext.Current.Server.MachineName + "\n";
        odpoved += "bazovy adresar: " + Directory.GetCurrentDirectory() + "\n";
        odpoved += "index je v adresari \"" + adresarIndex + "\"\n";
        odpoved += "databaza: server -> " + ConfigurationManager.AppSettings["dbServer"];
        odpoved += ", nazov -> " + ConfigurationManager.AppSettings["dbNazov"] + " ... ";
        if (db.otestujSpojenie()) odpoved += "spojenie funkcne\n";
        else odpoved += "neexistuje spojenie s databazou\n";

        zapisDoLogu(odpoved);
        return odpoved;
    }

    // vrati nastavenia ciest k log suborom a nazov indexu
    [WebMethod]
    public string[] nastavenia()
    {
        string[] nastavenia = { cestaLogSuboru, cestaLogFoobe, adresarIndex };
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : nastavenia\n");
        return nastavenia;
    }

    // nastavenie cesty k indexu
    [WebMethod]
    public void zmenAdresarIndexu(string novyAdresar)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : zmenAdresarIndexu(" + novyAdresar + ")\n");

        XmlDocument xmlDoc = new XmlDocument();
        xmlDoc.Load("web.config");
        XmlNode appSettingsNode = xmlDoc.SelectSingleNode("configuration/appSettings");

        foreach (XmlNode childNode in appSettingsNode)
        {
            if (childNode.Attributes["key"].Value == "adresarIndex")
                childNode.Attributes["value"].Value = novyAdresar;
        }
        xmlDoc.Save("web.config");
        xmlDoc.Save(AppDomain.CurrentDomain.SetupInformation.ConfigurationFile);

        adresarIndex = novyAdresar;
    }

    // nastavenie nastaveni databazy
    [WebMethod]
    public void zmenNastaveniaDatabazy(string databazovyServer, string databaza, string login, string heslo)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : zmenNastaveniaDatabazy(" + databazovyServer + ", " + databaza + ", " + login + ", " + heslo + ")");

        XmlDocument xmlDoc = new XmlDocument();
        xmlDoc.Load("web.config");
        XmlNode appSettingsNode = xmlDoc.SelectSingleNode("configuration/appSettings");

        foreach (XmlNode childNode in appSettingsNode)
        {
            if (childNode.Attributes["key"].Value == "dbServer")
                childNode.Attributes["value"].Value = databazovyServer;
            if (childNode.Attributes["key"].Value == "dbNazov")
                childNode.Attributes["value"].Value = databaza;
            if (childNode.Attributes["key"].Value == "dbLogin")
                childNode.Attributes["value"].Value = login;
            if (childNode.Attributes["key"].Value == "dbHeslo")
                childNode.Attributes["value"].Value = heslo;
        }
        xmlDoc.Save("web.config");
        xmlDoc.Save(AppDomain.CurrentDomain.SetupInformation.ConfigurationFile);
    }

    // nastavenie ciest k log suborom a indexu - logy su pevne dane
    /*[WebMethod]
    public void zmenitNastavenia(string novaCestaLogSuboru, string novaCestaLogFoobe, string novaCestaIndex)
    {
        if(novaCestaLogSuboru.Length != 0)cestaLogSuboru = novaCestaLogSuboru;
        if (novaCestaLogFoobe.Length != 0) cestaLogFoobe= novaCestaLogFoobe;
        if (novaCestaIndex.Length != 0) cestaIndex = novaCestaIndex;
    }*/

    //
    //****** nastavenie databazy  ***********************************************************//
    //

    // vratenie nastaveni databazy, tj. server, nazovDB...login, heslo nie
    [WebMethod]
    public string[] vratNastaveniaDB()
    {
        return new string[] {ConfigurationManager.AppSettings["dbServer"],
                ConfigurationManager.AppSettings["dbNazov"]};
    }


    //
    //****** vyhladavaci modul **************************************************************//
    //

    // vratenie poctu dokumentov v indexe
    // -1 chyba
    [WebMethod]
    public int vratPocetDokVIndexe()
    {
        int pocet = foobe.vratPocetDokVIndexe();
        zapisDoLogu(DateTime.Now + " -> vratPocetDokVIndexe ... " + pocet + "\n");

        return pocet;
    }

    // vyhladanie dokumentu s id zadanym v argumente
    [WebMethod]
    public Vysledok vyhladajDokPodlaId(int id)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : vyhladajDokPodlaId (" + id + ") ... ");

        Dokument dokument = foobe.vyhladajDokPodlaId(id);

        if (dokument == null)
        {
            zapisDoLogu("nenajdeny\n");
            return new Vysledok();
        }

        zapisDoLogu("ok\n");

        Vysledok vysledok = new Vysledok();
        vysledok.id = dokument.VratID();
        vysledok.nazov = dokument.VratNazov();
        vysledok.subor = dokument.VratSubor();
        vysledok.autor = dokument.VratAutora();
        vysledok.kategoria = dokument.VratKategoriu();
        vysledok.relevancia = dokument.VratRelevanciu();
        vysledok.sposobVyhladania = "0000";

        zapisDoLogu("nazov: " + vysledok.nazov + "\nautor: " + vysledok.autor +
            "\nkategoria: " + vysledok.kategoria + "\nsubor: " + vysledok.subor + "\n\n");

        return vysledok;
    }

    // vyhladanie dokumentov podla dotazu
    [WebMethod]
    public Vysledok[] vyhladajDokPodlaDotazu(string dotaz)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : vyhladajDokPodlaDotazu: " + dotaz + "\n");

        Dokument[] dokumenty = foobe.vyhladajDokPodlaDotazu(dotaz);

        if (dokumenty == null)
        {
            zapisDoLogu("chyba\n");
            return null;
        }

        zapisDoLogu("najdenych " + dokumenty.Length + "\n");

        Vysledok[] vysledky = new Vysledok[dokumenty.Length];
        for (int i = 0; i < dokumenty.Length; i++)
        {
            vysledky[i].id = dokumenty[i].VratID();
            vysledky[i].nazov = dokumenty[i].VratNazov();
            vysledky[i].subor = dokumenty[i].VratSubor();
            vysledky[i].autor = dokumenty[i].VratAutora();
            vysledky[i].kategoria = dokumenty[i].VratKategoriu();
            vysledky[i].relevancia = dokumenty[i].VratRelevanciu();
            vysledky[i].sposobVyhladania = "1111";
        }

        return vysledky;
    }

    //
    //******  pridavanie, odstranovanie a stahovanie dokumentov  *****************************
    //

    // pridanie dokumentu...nahratie na server, zindexovanie, zapis do databazy
    // 0 ok
    // -1 chyba pri vytvarani suboru
    // -2 nesuhlasi checksuma
    // -3 chyba pri pridavani do indexu
    // -4 chyba pri pridavani do db

    [WebMethod]
    public int pridajDokument(string nazov, string autor, string subor, string kategoria, byte[] prud, byte[] checksuma)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : pridajDokument(" + nazov + ", " + autor + ", " + subor + ", " + kategoria + ")\n");
        try
        {
            FileStream fs = new FileStream("App_Data\\" + subor, FileMode.Create);
            fs.Write(prud, 0, prud.Length);
            fs.Dispose();
            fs.Close();
        }
        catch (Exception e)
        {
            zapisDoLogu("chyba pri vytvarani suboru\n" + e.ToString() + "\n");
            return -1;
        }

        // overenie checksumy
        MD5 md5 = MD5.Create();
        byte[] vypocitanaChecksuma = md5.ComputeHash(File.ReadAllBytes("App_Data\\" + subor));

        zapisDoLogu("prijata checksuma:    " + Checksuma.checksumaDoString(checksuma) + "\nvypocitana checksuma: " + Checksuma.checksumaDoString(vypocitanaChecksuma) + "\n");

        if (!Checksuma.porovnajChecksumy(vypocitanaChecksuma, checksuma))
        {
            zapisDoLogu("-> nezhodne checksumy\n");
            return -2;
        }

        zapisDoLogu("-> zhodne checksumy\n");
        md5.Clear();

        // zaindexovanie
        int id = foobe.pridajDokDoIndexu(nazov, autor, kategoria, subor, "App_Data\\");

        if (id == -1)
        {
            zapisDoLogu("-> chyba pri pridavani do indexu\n");
            return -3;
        }

        zapisDoLogu("-> pridanie do indexu uspesne...priradene id " + id + "\n");

        // pridanie suboru do databazy
        try
        {
            db.vlozSubor(id, nazov, autor, kategoria, File.ReadAllBytes("App_Data\\" + subor));

            // otestovanie ci ulozilo
            if (db.existujeDokument(id))
                zapisDoLogu("-> pridanie do databazy uspesne\n");
            else
            {
                zapisDoLogu("-> dokument nebol pridany do databazy\n");
                return -4;
            }
        }
        catch (Exception e)
        {
            zapisDoLogu("-> chyba pri pridavani do databazy\n" + e.ToString() + "\n");
            return -4;
        }

        // odstranenie docasneho suboru
        try
        {
            File.Delete("App_Data\\" + subor);
            zapisDoLogu("-> odstranenie docasneho suboru uspesne\n");
        }
        catch (Exception e)
        {
            zapisDoLogu("-> chyba pri odstranovani docasneho suboru\n" + e.ToString());
        }

        // pridanie prepojeni
        try
        {
            Dokument dokument = foobe.vyhladajDokPodlaId(id);
            Prepojenia prepojenia = new Prepojenia();
            prepojenia.najdiPrepojenia(dokument, foobe, db);
            zapisDoLogu("-> vytvaranie prepojeni dokumentu ukoncene\n");
        }
        catch (Exception e)
        {
            zapisDoLogu("-> chyba pri vytvarani prepojeni\n" + e.ToString());
        }

        zapisDoLogu("pridanie dokumentu bolo uspesne\n");
        return 0;   // v poriadku
    }

    // stiahnutie suboru ku klientovi
    [WebMethod]
    public byte[] stiahniDokument(string login, int idDokumentu)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " -> " + login + " : stiahniDokument s id " + idDokumentu + "\n");

        // priradenie dokumentu pouzivatelovi - modul socialnych sieti
        db.pridajDokumentPouzivatelovi(idDokumentu, db.vratPouzivatelaPodlaLoginu(login).ID);

        try
        {
            return db.vyberSubor(idDokumentu).subor;
        }
        catch (Exception e)
        {
            zapisDoLogu("-> chyba: " + e.ToString());
            return null;
        }
    }

    // odstranenie dokumentu z indexu aj z databazy
    // 0 ok
    // 1 id nie je v indexe
    // -1 chyba
    [WebMethod]
    public int odstranDokId(int id)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : odstranDokId (" + id + ")\n");
        int vysledok = foobe.odstranDokId(id);

        if (vysledok == 0)
        {
            zapisDoLogu("-> uspesne odstranenie z indexu\n");
            try
            {
                db.odstranSubor(id);
                zapisDoLogu("-> uspesne odstranenie z databazy\n");
            }
            catch (Exception e)
            {
                zapisDoLogu("-> chyba pri odstranovani z databazy!!!\n" + e.ToString() + "\n");
                return -1;
            }

            return 0;
        }
        else if (vysledok == 1)
        { zapisDoLogu("nie je v indexe\n"); return 1; }
        else
        { zapisDoLogu("-> chyba pri odstranovani z indexu\n"); return -1; }
    }

    //
    //******  databazove funkcionality  ***************************************
    //

    // overenie loginu a hesla
    [WebMethod]
    public bool overLogin(string login, string heslo)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : overLogin (" + login + ") -> ");
        bool vysledok = db.overLogin(login, heslo);

        zapisDoLogu(vysledok + "\n");
        return vysledok;
    }

    // pridanie pouzivatela
    [WebMethod]
    public bool pridajPouzivatela(string login, string heslo)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : pridajPouzivatela (" + login + ") -> ");
        bool vysledok = db.pridajPouzivatela(login, heslo);

        zapisDoLogu(vysledok + "\n");
        return vysledok;
    }

    // odstranenie pouzivatela
    [WebMethod]
    public bool odstranPouzivatela(string login)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : odstranPouzivatela (" + login + ") -> ");
        bool vysledok = db.odstranPouzivatela(login);

        zapisDoLogu(vysledok + "\n");
        return vysledok;
    }

    // vratenie vah modulov
    [WebMethod]
    public float[] vratVahyModulov(string login)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : vratVahyModulov (" + login + ") -> ");
        if (!db.existujePouzivatel(login))
        {
            zapisDoLogu("login neexistuje\n");
            return null;
        }
        float[] vysledok = db.vratVahyModulov(login);

        zapisDoLogu(vysledok[0] + ", " + vysledok[1] + ", " + vysledok[2] + ", " + vysledok[3] + "\n");
        return vysledok;
    }

    // vratenie vah kategorii
    [WebMethod]
    public string vratVahyKategorii(string login)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : vratVahyKategorii (" + login + ")\n");
        if (!db.existujePouzivatel(login))
        {
            zapisDoLogu("login neexistuje\n");
            return "";
        }
        string vysledok = db.vratVahyKategorii(login);

        //zapisDoLogu("\n" + vysledok + "\n");
        return vysledok;
    }

    // zmena vah modulov
    [WebMethod]
    public bool zmenVahyModulov(string login, string vaha1, string vaha2, string vaha3, string vaha4)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : zmenVahyModulov (" + login + ", " + vaha1 + ", " + vaha2 + ", " + vaha3 + ", " + vaha4 + ") -> ");
        bool vysledok = db.zmenVahyModulov(login, vaha1, vaha2, vaha3, vaha4);

        zapisDoLogu(vysledok + "\n");
        return vysledok;
    }



    // zmena vah kategorii
    [WebMethod]
    public bool zmenVahyKategorii(string login, string noveVahy)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : zmenVahyKategorii (" + login + ", '" + noveVahy + "') -> ");
        bool vysledok = db.zmenVahyKategorii(login, noveVahy);

        zapisDoLogu(vysledok + "\n");
        return vysledok;
    }

    //
    //******  vyhladanie dokumentov kombinaciou modulov  **********************
    //

    // vyhladanie dokumentov jednotlivymi modulmi a kombinacia vysledkov
    [WebMethod]
    public Vysledok[] vyhladajDokumenty(string klucoveSlova, string kategoria, string login, string moduly) // moduly (klucoveSlova,kategorie,prepojenia,pouzivatelia)
    {
        zapisDoLogu(DateTime.Now + " " + HttpContext.Current.Request.UserHostAddress + " : vyhladajDokumenty (" + klucoveSlova + ", " + kategoria + ", " + login + ", " + moduly + ") -> ");
        if (moduly.Length < 4)
        {
            zapisDoLogu("zly format stringu moduly\n");
            return null;
        }

        bool modulVyhladavanie = false; VysledokModulu[] vysledkyVyhladavanie = new VysledokModulu[0];
        bool modulKategorie = false; VysledokModulu[] vysledkyKategorie = new VysledokModulu[0];
        bool modulPrepojenia = false; VysledokModulu[] vysledkyPrepojenia = new VysledokModulu[0];
        bool modulPouzivatelia = false; VysledokModulu[] vysledkyPouzivatelia = new VysledokModulu[0];

        if (moduly[0] == '1') modulVyhladavanie = true;
        if (moduly[1] == '1') modulKategorie = true;
        if (moduly[2] == '1') modulPrepojenia = true;
        if (moduly[3] == '1') modulPouzivatelia = true;

        // modul vyhladavanie
        Dokument[] dokumenty = new Dokument[0];
        if (modulVyhladavanie)
        {
            if (klucoveSlova.Length != 0)  // musia byt zadane klucove slova
            {
                dokumenty = foobe.vyhladajDokPodlaDotazu(klucoveSlova);
                if (dokumenty == null)
                {
                    dokumenty = new Dokument[0];
                    zapisDoLogu("chyba pri vyhladavani dokumentov vo vyhladavacom module\n");
                }

                vysledkyVyhladavanie = new VysledokModulu[dokumenty.Length];
                for (int i = 0; i < dokumenty.Length; i++)
                {
                    vysledkyVyhladavanie[i].id = dokumenty[i].VratID();
                    vysledkyVyhladavanie[i].relevancia = dokumenty[i].VratRelevanciu();
                }
            }
        }

        // modul prepojenia
        if (modulPrepojenia && dokumenty.Length != 0)
        {
            Prepojenia prepojenia = new Prepojenia();
            vysledkyPrepojenia = prepojenia.vratDokumenty(dokumenty, db);
        }
        // modul pouzivatelia
        if (modulPouzivatelia)
        {
            Recommender recommender = new Recommender(db.vratIdPouzivatela(login), @"Data source = " + ConfigurationManager.AppSettings["dbServer"] + "; database = " + ConfigurationManager.AppSettings["dbNazov"] + "; User ID = " + ConfigurationManager.AppSettings["dbLogin"] + "; Password = " + ConfigurationManager.AppSettings["dbHeslo"] + ";");
            vysledkyPouzivatelia = recommender.RecommendDocs(10, 10);
        }

        #region
        
        // vypocet relevancii dokumentov
        List<RelevancieDokumentu> relevancie = new List<RelevancieDokumentu>();
        int tmp; RelevancieDokumentu r; bool najdeny; int pocet;
        for (tmp = 0; tmp < vysledkyVyhladavanie.Length; tmp++)  //vyhladavanie
        {
            r = new RelevancieDokumentu();
            r.id = vysledkyVyhladavanie[tmp].id;
            r.r1 = vysledkyVyhladavanie[tmp].relevancia;
            r.r2 = 0f;
            r.r3 = 0f;
            relevancie.Add(r);
        }

        for (tmp = 0; tmp < vysledkyPrepojenia.Length; tmp++)    //prepojenia
        {
            najdeny = false;
            pocet = relevancie.Count;
            for (int tmp2 = 0; tmp2 < pocet; tmp2++)
            {
                if (relevancie[tmp2].id == vysledkyPrepojenia[tmp].id)
                {
                    relevancie[tmp2].r2 = vysledkyPrepojenia[tmp].relevancia;
                    najdeny = true;
                }
            }
            if (najdeny) continue;
            // ak nenasiel tak prida
            r = new RelevancieDokumentu();
            r.id = vysledkyPrepojenia[tmp].id;
            r.r1 = 0f;
            r.r2 = vysledkyPrepojenia[tmp].relevancia;
            r.r3 = 0f;
            relevancie.Add(r);

        }

        for (tmp = 0; tmp < vysledkyPouzivatelia.Length; tmp++)    //pouzivatelia
        {
            najdeny = false;
            pocet = relevancie.Count;
            for (int tmp2 = 0; tmp2 < pocet; tmp2++)
            {
                if (relevancie[tmp2].id == vysledkyPouzivatelia[tmp].id)
                {
                    relevancie[tmp2].r3 = vysledkyPouzivatelia[tmp].relevancia;
                    najdeny = true;
                }
            }
            if (najdeny) continue;

            //ak nenasiel tak prida
            r = new RelevancieDokumentu();
            r.id = vysledkyPouzivatelia[tmp].id;
            r.r1 = 0f;
            r.r2 = 0f;
            r.r3 = vysledkyPouzivatelia[tmp].relevancia;
            relevancie.Add(r);
        }

        //zapisDoLogu("\nnajdeneDokumenty a ich relevancie podla modulov:\n");
        //foreach (RelevancieDokumentu rel in relevancie) zapisDoLogu(rel.id + " " + rel.r1 + " " + rel.r2 + " " + rel.r3 + "\n");
        #endregion

        //VysledokModulu[] celkoveVysledky = vysledkyVyhladavanie;
        Dokument dokument; string sposob;
        Vysledok[] vratitVysledky = new Vysledok[relevancie.Count];
        for (int i = 0; i < vratitVysledky.Length; i++)
        {
            dokument = foobe.vyhladajDokPodlaId(relevancie[i].id);
            if (dokument == null)
            {
                zapisDoLogu("problem pri preberani vysledkov dokumentu s id: " + relevancie[i].id + "\n");
                continue;
            }
            vratitVysledky[i].id = dokument.VratID();
            vratitVysledky[i].nazov = dokument.VratNazov();
            vratitVysledky[i].autor = dokument.VratAutora();
            vratitVysledky[i].subor = dokument.VratSubor();
            vratitVysledky[i].kategoria = dokument.VratKategoriu();
            ///////////
            // vzorec
            ///////////

            float [] vahyModulov = db.vratVahyModulov(login);

            vratitVysledky[i].relevancia =  vahyModulov[0] * relevancie[i].r1 + 
                                            vahyModulov[2] * relevancie[i].r2 + 
                                            vahyModulov[3] * relevancie[i].r3;
            
            sposob = "";    // urcenie sposobu vyhladania
            if (relevancie[i].r1 != 0f) sposob = String.Concat(sposob, "1"); else sposob = String.Concat(sposob, "0");
            if (modulKategorie) sposob = String.Concat(sposob, "1"); else sposob = String.Concat(sposob, "0");
            if (relevancie[i].r2 != 0f) sposob = String.Concat(sposob, "1"); else sposob = String.Concat(sposob, "0");
            if (relevancie[i].r3 != 0f) sposob = String.Concat(sposob, "1"); else sposob = String.Concat(sposob, "0");
            
            vratitVysledky[i].sposobVyhladania = sposob;
        }

        // modul kategorie
        if (modulKategorie)
        {
            Kategorie kategorie = new Kategorie();
            vratitVysledky = kategorie.vypocetRelevancie(kategoria, db.vratVahyKategorii(login), vratitVysledky);
        }

        zapisDoLogu("najdenych " + vratitVysledky.Length + " dokumentov\n");
        return vratitVysledky;
    }

    //
    //******  log  ************************************************************
    //

    // metoda na pripisovanie do logu
    private void zapisDoLogu(string text)
    {
        File.AppendAllText(cestaLogSuboru, text);
    }
}

/// <summary>
/// Trieda na spracovanie najdenych dokumentov do celkovych vysledkov.
/// Dokumentu sa priradia vahy v jednotlivych moduloch, z ktorych sa vypocita
/// celkova relevancia dokumentu.
/// </summary>
class RelevancieDokumentu
{
    public int id;
    public float r1;
    public float r2;
    public float r3;

    public void nastavR1(float r)
    {
        r1 = r;
    }
    public void nastavR2(float r)
    {
        r3 = r;
    }
    public void nastavR3(float r)
    {
        r3 = r;
    }
}