/***********************************************************************
* Nazov suboru: Administracia.cs
* Autor: Zdenko Porubcan
* Datum vytvorenia: 16.12.2007
* 
* Rozhranie na administrativne ucely 
* (pridavanie/odstranovanie pouzivatelov, pridavanie/odstranovanie dokumetov)
***********************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections;
using System.Security.Cryptography;
using Foobe.FoobeServis;
using Foobe;
using System.Threading;

namespace Foobe
{
  // trieda formular na pridavanie a odstranovanie dokumentov
  public partial class Administracia : Form
  {
    System.IO.FileInfo vstupSubor;
    FoobeServis.FoobeServis servis;
    Progress pr;
    Vysledok[] pole;

    // KONSTRUKTOR
    public Administracia()
    {
      InitializeComponent();
      VyplnComboBox();
      servis = new FoobeServis.FoobeServis();
    }

    // pridanie pouzivatela
    private void btn_pouz_Click(object sender, EventArgs e)
    {
      string login = txb_login.Text;
      string passwd = txb_passwd.Text;
      string passwd2 = txb_passwd2.Text;
      servis = new FoobeServis.FoobeServis();

      if (!passwd.Equals(passwd2))
      {
        MessageBox.Show(this, "Nerovnos hesiel.", "Chyba!", MessageBoxButtons.OK);
        return;
      }

      bool b = servis.pridajPouzivatela(login, passwd);

      if ((login.Equals("")) && (passwd.Equals("")) && (passwd2.Equals("")))
      {
        MessageBox.Show(this, "Musia by vyplnen vetky polia!", "Chyba!", MessageBoxButtons.OK);
        return;
      }

      if (b)
      {
        MessageBox.Show(this, "Pouvate bol spene pridan.", "Pridanie pouvatea", MessageBoxButtons.OK);
        txb_login.Text = "";
        txb_passwd.Text = "";
        txb_passwd2.Text = "";
      }
      else MessageBox.Show(this, "Pouvate u existuje.", "Chyba!", MessageBoxButtons.OK);
    }

    // Ostranenie pouzivatela
    private void button1_Click(object sender, EventArgs e)
    {
      string login = txb_login.Text;
      servis = new FoobeServis.FoobeServis();

      if (login.Equals(""))
      {
        MessageBox.Show(this, "Nebolo zadan pouvatesk meno!", "Chyba!", MessageBoxButtons.OK);
        return;
      }

      if (servis.odstranPouzivatela(login))
      {
        txb_login.Text = "";
        txb_passwd.Text = "";
        txb_passwd2.Text = "";
        MessageBox.Show(this, "Pouvate " + login + " bol spene odstrnen.", "Odstrnenie pouvatea.", MessageBoxButtons.OK);
        return;
      }
      else 
      {
        MessageBox.Show(this, "Pouvate " + login + " sa nenaiel!", "Chyba!", MessageBoxButtons.OK);
        return;
      }
    }

    // Vyplnenie Comboboxu kategoriami
    public void VyplnComboBox()
    {
      ZozKategorii zoz = new ZozKategorii();
      string[] nazvy = zoz.vratZoznam();
      servis = new FoobeServis.FoobeServis();

      for (int i = 0; i < nazvy.Length; i++)
      {
        cbxKategoria.Items.Add(nazvy[i]);
      }

      cbxKategoria.SelectedIndex = 0;
    }

    // pridanie noveho suboru 
    private void btnPridajClick(object sender, EventArgs e)
    {
      string nazov = tbxNazov.Text;
      string autor = tbxAutor.Text;
      string kategoria;
      string cesta = tbxSubor.Text;
      servis = new FoobeServis.FoobeServis();

      if (cbxKategoria.Text.Trim().Equals("")) kategoria = "Nespecifikovana";
      else kategoria = cbxKategoria.Text;
      
      if (File.Exists(cesta))
      {
        FileStream st;
        try
        {
          st = new FileStream(cesta, FileMode.Open);
        }
        catch (System.IO.IOException)
        {
          MessageBox.Show(this, "Zvolen sbor je neprstupn, pretoe ho pouva in aplikcia.", "Chyba!", MessageBoxButtons.OK);
          return;
        }
        byte[] buffer = new byte[st.Length];
        st.Read(buffer, 0, (int)st.Length);
        st.Close();

        // vypocitanie checksumy
        MD5 md5 = MD5.Create();
        byte[] checksuma = md5.ComputeHash(buffer);
        md5.Clear();

        Thread thread = new Thread(vlakno);
        thread.Start();
        this.Enabled = false;

        int vysledok = servis.pridajDokument(nazov, autor, Path.GetFileName(cesta), kategoria, buffer, checksuma);

        pr.Close();
        thread.Abort();
        this.Enabled = true;

        if (vysledok == 0) // ok
        {
          //dialog o uspesnosti pridania
          string message = "Dokument bol spene pridan.\n\nNzov:\t  " + nazov + "\nAutor:\t  " +
                            autor + "\nKategria: " + kategoria + "\nSbor:\t  " + cesta;
          string caption = "Pridanie dokumentu";

          MessageBox.Show(this, message, caption, MessageBoxButtons.OK);
          tbxAutor.Text = "";
          tbxNazov.Text = "";
          tbxSubor.Text = "";
          cbxKategoria.SelectedIndex = 0;
        }
        else if(vysledok == -1)
          MessageBox.Show(this, "Pri pridvan sboru na server nastala chyba. Ak sa problm opakuje, kontaktujte sprvcu systmu.", "Chyba!", MessageBoxButtons.OK);
        else if(vysledok == -2)
          MessageBox.Show(this, "Pri pridvan sboru na server nastala chyba - prenesen sbor nie je identick s posielanm. Ak sa problm opakuje, kontaktujte sprvcu systmu.", "Chyba!", MessageBoxButtons.OK);
        else if(vysledok == -3)
          MessageBox.Show(this, "Pri pridvan sboru na server nastala chyba - sbor sa nepodarilo zaindexova. Ak sa problm opakuje, kontaktujte sprvcu systmu.", "Chyba!", MessageBoxButtons.OK);
        else if (vysledok == -4)
          MessageBox.Show(this, "Pri pridvan sboru do databzy nastala chyba. Ak sa problm opakuje, kontaktujte sprvcu systmu.", "Chyba!", MessageBoxButtons.OK);
      }
      else MessageBox.Show(this, "Neplatn cesta k sboru.", "Chyba!", MessageBoxButtons.OK);
    }

    // vlakno pre novy proces
    public void vlakno()
    {
      Application.Run(pr = new Progress(1));
    }

    // vyhladanie suborov
    private void btnVyhladajSuborClick(object sender, EventArgs e)
    {
      OpenFileDialog openFileDialog1 = new OpenFileDialog();
      servis = new FoobeServis.FoobeServis();

      openFileDialog1.InitialDirectory = "C:\\";
      openFileDialog1.Filter = "Dokumenty (*.txt, *.doc, *.docx, *.xls, *.xlsx, *.ppt*, *.rtf, *.pdf, *.htm, *.html)|*.txt;*.doc;*.docx;*.xls;*.xlsx;*.ppt;*.rtf;*.pdf;*.htm;*.html";

      openFileDialog1.FilterIndex = 1;
      openFileDialog1.RestoreDirectory = true;
      openFileDialog1.Multiselect = false;
      openFileDialog1.DereferenceLinks = false;
      openFileDialog1.Title = "Vybra dokument";

      if (openFileDialog1.ShowDialog() == DialogResult.OK) {
        tbxSubor.Text = openFileDialog1.FileName;
        vstupSubor = new FileInfo(openFileDialog1.FileName);
      }
    }

    // odstranenie oznacenych dokumentov
    private void btnOdstranDokumentyClick(object sender, EventArgs e)
    {
      DataGridViewSelectedRowCollection rc = tabDokumentov.SelectedRows;
      servis = new FoobeServis.FoobeServis();
      int n;

      for (int i = 0; i < rc.Count; i++)
      {
        DataGridViewRow row = rc[i];
        n = servis.odstranDokId(Int32.Parse(row.Cells[4].Value.ToString()));
        tabDokumentov.Rows.Remove(row);
      }
    }
    
    // vyhladanie dokumentov
    private void btnVyhladajKlucoveSlovaClick(object sender, EventArgs e)
    {
      string klucoveSlova;
      servis = new FoobeServis.FoobeServis();

      klucoveSlova = tbxOdstranPodlaKlucoveSlova.Text;

      if (klucoveSlova.Trim().Length == 0) return;

      try { pole = servis.vyhladajDokPodlaDotazu(klucoveSlova); }
      catch (System.Net.WebException)
      {
        MessageBox.Show(this, "Nepodarilo sa nadviaza spojenie so serverom.", "Chyba!", MessageBoxButtons.OK);
        return;
      }
      catch (System.InvalidOperationException)
      {
        MessageBox.Show(this, "Servis nepristupny.", "Chyba!", MessageBoxButtons.OK);
        return;
      }

      if (pole == null)
      {
        MessageBox.Show(this, "Nekorektn dotaz.", "Chyba!", MessageBoxButtons.OK);
        return;
      }

      ZapisDoTabulky(pole);
    }
    
    // zapisanie najdenych dokumentov do tabulky
    public void ZapisDoTabulky(Vysledok[] pole) 
    {
      //vymazanie tabulky
      tabDokumentov.DataSource = null;
      tabDokumentov.Rows.Clear();
      tabDokumentov.Columns.Clear();

      tabDokumentov.Columns.Add("st0", "Nzov dokumentu");
      tabDokumentov.Columns.Add("st1", "Autor");
      tabDokumentov.Columns.Add("st2", "Kategria");
      tabDokumentov.Columns.Add("st3", "Sbor");
      tabDokumentov.Columns.Add("st4", "id");

      tabDokumentov.Columns[0].Width = 180;
      tabDokumentov.Columns[1].Width = 100;
      tabDokumentov.Columns[2].Width = 100;
      tabDokumentov.Columns[3].Width = 100;
      tabDokumentov.Columns[4].Width = 20;

      tabDokumentov.Columns[4].Visible = false;

      for (int i = 0; i < pole.Length; i++)
      {
        tabDokumentov.Rows.Add();
        tabDokumentov.Rows[i].Cells[0].Value = pole[i].nazov;
        tabDokumentov.Rows[i].Cells[1].Value = pole[i].autor;
        tabDokumentov.Rows[i].Cells[2].Value = pole[i].kategoria;
        tabDokumentov.Rows[i].Cells[3].Value = pole[i].subor;
        tabDokumentov.Rows[i].Cells[4].Value = pole[i].id;
      }
    }

    // main
    static void Main()
    {
      //Kategorie k = new Kategorie();
      //k.NacitanieSuboru();
      Application.Run(new Administracia());
    }

    private void nastavenieURLServeraToolStripMenuItem_Click_1(object sender, EventArgs e)
    {
      NastavenieServera ns = new NastavenieServera(servis);
      ns.Visible = true;
    }
  }
}