/***********************************************************************
* Nazov suboru: NastavenieKategorii.cs
* Autor: Zdenko Porubcan
* Datum vytvorenia: 21.03.2008
* 
* Rozhranie na nastavenie vah podobnosti jednotlivych kategorii dokumentu
***********************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.IO;
using Foobe.FoobeServis;

namespace Foobe
{
  public partial class NastavenieKategorii : Form
  {
    FoobeServis.FoobeServis servis = new FoobeServis.FoobeServis();
    ArrayList nazvyKategorii = new ArrayList();
    ZozKategorii zoz = new ZozKategorii();
    float[,] matica;
    string vahy;
    string login;

    // KONSTRUKTOR
    public NastavenieKategorii(string l)
    {
      login = l;
      InitializeComponent();
      NacitanieKategorii();

      tabKategorie.DataSource = null;
      tabKategorie.Rows.Clear();
      tabKategorie.Columns.Clear();

      int i, j;
      for (i = 0; i < nazvyKategorii.Count; i++)
      {
        tabKategorie.Columns.Add("st" + i.ToString(), nazvyKategorii[i].ToString());
        tabKategorie.Columns[i].Width = 80;
        tabKategorie.Rows.Add();
        tabKategorie.Rows[i].HeaderCell.Value = nazvyKategorii[i].ToString();
        tabKategorie.Rows[i].Height = 35;
      }

      for (i = 0; i < nazvyKategorii.Count; i++)
      {
        for (j = 0; j < nazvyKategorii.Count; j++)
        {
          if (j <= i) tabKategorie.Rows[i].Cells[j].Value = matica[i, j];
          else
          {
            tabKategorie[j, i].Style.SelectionBackColor = Color.Gray;
            tabKategorie[j, i].Style.BackColor = Color.Gray;
            tabKategorie[j, i].ReadOnly = true;
          }
        }
      }
    }

    // metoda na nacitanie kategorii
    public void NacitanieKategorii()
    {
      string[] nazvy;
      string[] pole;
      char[] oddelovac = { ' ' };

      nazvy = zoz.vratZoznam();

      try
      {
        for (int i = 0; i < nazvy.Length; i++)
        {
          nazvyKategorii.Add(nazvy[i]);
        }

        matica = new float[nazvyKategorii.Count, nazvyKategorii.Count];

        vahy = servis.vratVahyKategorii(login);

        int poc = 0, x, y;
        float w;

        vahy.Trim();
        pole = vahy.Split(oddelovac);
        
        for (poc = 0; poc < pole.Length; poc += 3) {
          x = Int32.Parse(pole[poc]);
          y = Int32.Parse(pole[poc+1]);
          w = float.Parse(pole[poc+2]);

          matica[x, y] = w;
          matica[y, x] = w;
        }
      }

      catch
      {
        MessageBox.Show(this, "Chyba pri nacitani hodnot!", "Chyba!", MessageBoxButtons.OK);
      }
    }

    // ulozenie hodnot
    private void button1_Click(object sender, EventArgs e)
    {
      if (!kontrola())
      {
        MessageBox.Show(this, "Nekorektne zadan hodnoty!", "Chyba!", MessageBoxButtons.OK);
        return;
      }

      int i, j;

      vahy = "";

      for (i = 0; i < nazvyKategorii.Count; i++)
      {
        for (j = 0; j <= i; j++)
        {
          vahy += i.ToString() + " " + j.ToString() + " " + tabKategorie.Rows[i].Cells[j].Value.ToString() + " "; 
        }
      }

      bool suUloz;
      suUloz = servis.zmenVahyKategorii(login, vahy.Trim());

      if (suUloz)
      {
        MessageBox.Show(this, "Vhy boli uloen", "OK", MessageBoxButtons.OK);
      }
      else
      {
        MessageBox.Show(this, "Chyba pri ukladan vh!", "Chyba!", MessageBoxButtons.OK);
        return;
      }
    }

    // kontrola hodnot
    private bool kontrola()
    {
      for (int i = 0; i < nazvyKategorii.Count; i++)
      {
        for (int j = 0; j <= i; j++)
        {
          try
          {
            float.Parse(tabKategorie[j, i].Value.ToString());
          }
          catch (Exception)
          {
            return false;
          }
          float a = float.Parse(tabKategorie[j, i].Value.ToString());
          if ((a < 0.0) || (a > 1.0)) return false;
        }
      }
      return true;
    }

    // zatvorenie okna
    private void button2_Click(object sender, EventArgs e)
    {
      this.Close();
    }

    // vratenie default hodnot
    private void default_btn_Click(object sender, EventArgs e)
    {
      string[] pole;
      char[] oddelovac = { ' ' };
      int i, j;

      try
      {
        vahy = zoz.vratDefaultVahy();

        int poc = 0, x, y;
        float w;

        vahy.Trim();
        pole = vahy.Split(oddelovac);

        for (poc = 0; poc < pole.Length; poc += 3)
        {
          x = Int32.Parse(pole[poc]);
          y = Int32.Parse(pole[poc + 1]);
          w = float.Parse(pole[poc + 2]);

          matica[x, y] = w;
          matica[y, x] = w;
        }
      }

      catch
      {
        MessageBox.Show(this, "Chyba pri nacitani hodnot!", "Chyba!", MessageBoxButtons.OK);
      }

      for (i = 0; i < nazvyKategorii.Count; i++)
      {
        for (j = 0; j < nazvyKategorii.Count; j++)
        {
          if (j <= i) tabKategorie.Rows[i].Cells[j].Value = matica[i, j];
          else
          {
            tabKategorie[j, i].Style.SelectionBackColor = Color.Gray;
            tabKategorie[j, i].Style.BackColor = Color.Gray;
            tabKategorie[j, i].ReadOnly = true;
          }
        }
      }
    }
  }
}