/***********************************************************************
* Nazov suboru: Vyhladavanie.cs
* Autor: Zdenko Porubcan
* Datum vytvorenia: 16.12.2007
* 
* Hlavne rozhranie na vyhladavanie dokumentov
***********************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Collections;
using System.IO;
using Foobe.FoobeServis;
using System.Configuration;
using System.Threading;

namespace Foobe
{
  public partial class Vyhladavanie : Form
  {
    FoobeServis.FoobeServis servis;
    Vysledok[] pole;
    byte[] subor;
    Progress pr;
    string login;
    string sposob;
    Napoveda napoveda;

    // KONSTRUKTOR
    public Vyhladavanie(string s)
    {
      login = s;

      try
      {
        servis = new FoobeServis.FoobeServis();
      }
      catch (UriFormatException) 
      {
        MessageBox.Show(this, "Zl formt URL servera.", "Chyba!", MessageBoxButtons.OK);
      }
      
      InitializeComponent();

      VyplnComboBox();
    }

    // update servisu
    public void updateServisu() {
      servis = new FoobeServis.FoobeServis();    
    }

    // vyplnenie Comboboxu kategoriami
    public void VyplnComboBox()
    {
      ZozKategorii zoz = new ZozKategorii();
      string[] nazvy = zoz.vratZoznam();

      for (int i = 1; i < nazvy.Length; i++)
      {
        cbxKategorie.Items.Add(nazvy[i]);
      }
      cbxKategorie.SelectedIndex = 0;
    }

    // nastavenie servera
    public void nastavServerClick(object sender, EventArgs e)
    {
      NastavenieServera ns = new NastavenieServera(servis);
      ns.Visible = true;
    }

    // nastavenie kategorii
    public void nastavKategorieClick(object sender, EventArgs e)
    {
      NastavenieKategorii nk = new NastavenieKategorii(login);
      nk.Visible = true;
    }

    // nastavenie vah
    private void menuItem5_Click(object sender, EventArgs e)
    {
      Vahy v = new Vahy(login);
      v.Visible = true;
    }

    // vyhladanie dokumentov
    private void CheckKeys(object sender, System.Windows.Forms.KeyPressEventArgs e)
    {
      if (e.KeyChar == (char)13)
      {
        vyhladajDokumenty();
      }
    }

    // vyhladanie dokumentov
    private void btnVyhladajClick(object sender, EventArgs e)
    {
      vyhladajDokumenty();
    }

    // zapnutie/vypnutie kategorii
    public void cbxNastavKategorieClick(object sender, EventArgs e)
    {
      if (chbxKategoria.Checked == true)
      {
        cbxKategorie.Enabled = true;
        chbxKlucoveSlova.Checked = true;
        chbxKlucoveSlova.Enabled = false;
      }
      else
      {
        cbxKategorie.Enabled = false;
        chbxKlucoveSlova.Enabled = true;
      }
    }

    // zapnutie/vypnutie prepojeni
    private void chbxPrepojenia_CheckedChanged(object sender, EventArgs e)
    {
      if (chbxPrepojenia.Checked == true)
      {
        chbxKlucoveSlova.Checked = true;
        chbxKlucoveSlova.Enabled = false;
      }
      else
      {
        chbxKlucoveSlova.Enabled = true;
      }
    }

    // odovzdanie klucovych slova na vyhladanie
    private void vyhladajDokumenty()
    {
      string klucoveSlova;

      klucoveSlova = tbxKlucoveSlova.Text;
      sposob = "";
      if (chbxKlucoveSlova.Checked == true) sposob += "1";
      else sposob += "0";

      if (chbxKategoria.Checked == true) sposob += "1";
      else sposob += "0";

      if (chbxPrepojenia.Checked == true) sposob += "1";
      else sposob += "0";

      if (chbxPouzivatelia.Checked == true) sposob += "1";
      else sposob += "0";

      if (klucoveSlova.Trim().Length == 0) return;
      
      try { pole = servis.vyhladajDokumenty(klucoveSlova, cbxKategorie.Text, login, sposob); }
      catch (System.Net.WebException)
      {
        MessageBox.Show(this, "Nepodarilo sa nadviaza spojenie so serverom.", "Chyba!", MessageBoxButtons.OK);
        return;
      }

      if (pole == null) 
      {
        MessageBox.Show(this, "Nekorektn dotaz.", "Chyba!", MessageBoxButtons.OK);
        return;
      }
      
      ZapisDoTabulky(pole);
    }

    // metoda na zapisanie dat do tabulky
    public void ZapisDoTabulky(Vysledok[] poleDokumentov)
    {
      //vymazanie tabulky
      tabVysledkyVyhladavania.DataSource = null;
      tabVysledkyVyhladavania.Rows.Clear();
      tabVysledkyVyhladavania.Columns.Clear();

      tabVysledkyVyhladavania.Columns.Add("st0", "Nzov dokumentu");
      tabVysledkyVyhladavania.Columns.Add("st1", "Autor");
      tabVysledkyVyhladavania.Columns.Add("st2", "Kategria");
      tabVysledkyVyhladavania.Columns.Add("st3", "Sbor");
      tabVysledkyVyhladavania.Columns.Add("st4", "Relevancia");
      tabVysledkyVyhladavania.Columns.Add("", "") ;
      tabVysledkyVyhladavania.Columns.Add("", "");
      tabVysledkyVyhladavania.Columns.Add("", "");
      tabVysledkyVyhladavania.Columns.Add("", "");
      tabVysledkyVyhladavania.Columns.Add("id", "");
      tabVysledkyVyhladavania.Columns.Add("sposob", "");

      tabVysledkyVyhladavania.Columns[0].Width = 180;
      tabVysledkyVyhladavania.Columns[1].Width = 100;
      tabVysledkyVyhladavania.Columns[2].Width = 100;
      tabVysledkyVyhladavania.Columns[3].Width = 100;
      tabVysledkyVyhladavania.Columns[4].Width = 65;
      tabVysledkyVyhladavania.Columns[5].Width = 10;
      tabVysledkyVyhladavania.Columns[6].Width = 10;
      tabVysledkyVyhladavania.Columns[7].Width = 10;
      tabVysledkyVyhladavania.Columns[8].Width = 10;
      tabVysledkyVyhladavania.Columns[4].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
      
      // zneviditelnenie stlpcov
      tabVysledkyVyhladavania.Columns[9].Visible = false;
      tabVysledkyVyhladavania.Columns[10].Visible = false;

      tabVysledkyVyhladavania.Columns[0].DividerWidth = 1;
      tabVysledkyVyhladavania.Columns[1].DividerWidth = 1;
      tabVysledkyVyhladavania.Columns[2].DividerWidth = 1;
      tabVysledkyVyhladavania.Columns[3].DividerWidth = 1;
      tabVysledkyVyhladavania.Columns[4].DividerWidth = 1;
      tabVysledkyVyhladavania.Columns[5].Resizable = DataGridViewTriState.False;
      tabVysledkyVyhladavania.Columns[6].Resizable = DataGridViewTriState.False;
      tabVysledkyVyhladavania.Columns[7].Resizable = DataGridViewTriState.False;
      tabVysledkyVyhladavania.Columns[8].Resizable = DataGridViewTriState.False;

      // naplnenie tabulky
      for (int i = 0; i < poleDokumentov.Length; i++)
      {
        tabVysledkyVyhladavania.Rows.Add();
        tabVysledkyVyhladavania.Rows[i].Cells[0].Value = poleDokumentov[i].nazov;
        tabVysledkyVyhladavania.Rows[i].Cells[1].Value = poleDokumentov[i].autor;
        tabVysledkyVyhladavania.Rows[i].Cells[2].Value = poleDokumentov[i].kategoria;
        tabVysledkyVyhladavania.Rows[i].Cells[3].Value = poleDokumentov[i].subor;
        tabVysledkyVyhladavania.Rows[i].Cells[4].Value = poleDokumentov[i].relevancia;
        tabVysledkyVyhladavania.Rows[i].Cells[9].Value = poleDokumentov[i].id;
        tabVysledkyVyhladavania.Rows[i].Cells[10].Value = poleDokumentov[i].sposobVyhladania;
        zobrazSposobVyhladania(poleDokumentov[i].sposobVyhladania, i);
      }

      // sort
      //tabVysledkyVyhladavania.Sort(tabVysledkyVyhladavania.Columns[4], ListSortDirection.Descending);
      if (rbtnRelevancia.Checked) sortRelevancia();
      else sortSposob();

      label5.Text = "Njdench " + poleDokumentov.Length + " dokumentov.";
    }
      
    // zobrazenie sposobu vyhladania
    private void zobrazSposobVyhladania(string sposob, int index) 
    {
      if (sposob[0] == '1') tabVysledkyVyhladavania[5, index].Style.BackColor = Color.Aqua;
      if (sposob[1] == '1') tabVysledkyVyhladavania[6, index].Style.BackColor = Color.Yellow;
      if (sposob[2] == '1') tabVysledkyVyhladavania[7, index].Style.BackColor = Color.Chocolate;
      if (sposob[3] == '1') tabVysledkyVyhladavania[8, index].Style.BackColor = Color.LimeGreen;
    }

    // sortovanie podla relevancie
    private void sortRelevancia()
    {
      tabVysledkyVyhladavania.Sort(tabVysledkyVyhladavania.Columns[4], ListSortDirection.Descending);
      tabVysledkyVyhladavania.ClearSelection();
    }

    // sortovanie podla sposobu
    private void sortSposob() 
    {
      tabVysledkyVyhladavania.Sort(tabVysledkyVyhladavania.Columns[10], ListSortDirection.Descending);
      tabVysledkyVyhladavania.ClearSelection();
    }

    // ulozenie dokument
    private void btnOtvoritClick(object sender, EventArgs e)
    {
      if (tabVysledkyVyhladavania.SelectedRows.Count <= 0)
      {
        MessageBox.Show(this, "Nebol vybrat iadny sbor.", "Chyba!", MessageBoxButtons.OK);
        return;
      }
      SaveFileDialog saveFileDialog1 = new SaveFileDialog();

      Thread thread = new Thread(vlakno);
      thread.Start();
      this.Enabled = false;
      
      subor = servis.stiahniDokument(login, Int32.Parse(tabVysledkyVyhladavania.SelectedRows[0].Cells[9].Value.ToString()));
      //Console.WriteLine(Int32.Parse(tabVysledkyVyhladavania.SelectedRows[0].Cells[9].Value.ToString()));


      pr.Close();
      thread.Abort();
      this.Enabled = true;

      saveFileDialog1.InitialDirectory = "C:\\";
      saveFileDialog1.Filter = "All Files|*.*";
      string s = tabVysledkyVyhladavania.SelectedRows[0].Cells[3].Value.ToString();
      saveFileDialog1.AddExtension = true;
      saveFileDialog1.FileName = s;
      saveFileDialog1.FilterIndex = 1;
      saveFileDialog1.RestoreDirectory = true;
      saveFileDialog1.Title = "Uloi dokument";
      
      if (saveFileDialog1.ShowDialog() == DialogResult.OK)
      {
            FileStream fs = new FileStream(saveFileDialog1.FileName, FileMode.Create);
            fs.Write(subor, 0, subor.Length);
            fs.Close();
      }
    }

    // otvorenie suboru
    private void button1_Click(object sender, EventArgs e)
    {
      if (tabVysledkyVyhladavania.SelectedRows.Count <= 0)
      {
        MessageBox.Show(this, "Nebol vybrat iadny sbor.", "Chyba!", MessageBoxButtons.OK);
        return;
      } 

      Process proces = new Process();
      String nazov = tabVysledkyVyhladavania.SelectedRows[0].Cells[3].Value.ToString();
      String cesta = System.IO.Path.GetTempPath() + nazov;

      Thread thread = new Thread(vlakno);
      thread.Start();
      this.Enabled = false;

      subor = servis.stiahniDokument(login, Int32.Parse(tabVysledkyVyhladavania.SelectedRows[0].Cells[9].Value.ToString()));

      pr.Close();
      thread.Abort();
      this.Enabled = true;

      FileStream fs = new FileStream(cesta, FileMode.Create);
      fs.Write(subor, 0, subor.Length);
      fs.Close();
      
      try
      {
        proces.StartInfo.FileName = cesta;
        proces.Start();
      }
      catch (FileNotFoundException)
      {
        MessageBox.Show(this, "Neplatn cesta k sboru: " + proces.StartInfo.FileName, "Chyba!", MessageBoxButtons.OK);
      }
      catch (System.ComponentModel.Win32Exception)
      {
        MessageBox.Show(this, "Neexistuje asocicia s tmto typom sboru: " + proces.StartInfo.FileName, "Chyba!", MessageBoxButtons.OK);
      }
    }

    // vlakno na novy proces
    public void vlakno()
    {
      Application.Run(pr = new Progress(-1));
    }

    // sort podla relevancie
    private void rbtnRelevancia_Click(object sender, EventArgs e)
    {
      try { sortRelevancia(); }
      catch (System.ArgumentOutOfRangeException) { }
    }

    // sort podla relevancie
    private void rbtnSposobVyhladavania_Click(object sender, EventArgs e)
    {
      try { sortSposob(); }
      catch (System.ArgumentOutOfRangeException) { }
    }

    // otvorenie napovedy
    private void help_btn_Click_1(object sender, EventArgs e)
    {
      napoveda = new Napoveda();
      napoveda.Visible = true;
    }

    // main
    static void Main()
    {
      Application.Run(new Vyhladavanie("pokus"));
    }
  }
}
